/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.impl.tools;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Set;

import javax.servlet.RequestDispatcher;
import javax.servlet.Servlet;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;

import org.ow2.opensuit.xml.interfaces.IValidationSupport;
import org.ow2.opensuit.xmlmap.XmlInstantiator;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.xml.sax.SAXException;


public class Validator
{
	public static void main(String[] args)
	{
		if(args.length < 2)
		{
			System.err.println("Usage: Validator <xml> <root web app> [<level>]");
			return;
		}
		
		String xmlFileName = args[0];
		String webRootDirName = args[1];
		String level = args.length > 2 ? args[2] : "w";
		int dumpLevel = IInitializationSupport.WARNING;
		if(level.startsWith("e") || level.startsWith("E"))
			dumpLevel = IInitializationSupport.ERROR;
		else if(level.startsWith("i") || level.startsWith("I"))
			dumpLevel = IInitializationSupport.INFO;
		
		// --- check xmlfile refers to an existing file
		File xmlFile = new File(xmlFileName);
		if(!xmlFile.exists())
		{
			System.err.println("xml file ["+xmlFileName+"] not found.");
			return;
		}
		
		// --- check webRoot exists
		File webRootDir = new File(webRootDirName);
		if(!webRootDir.exists())
		{
			System.err.println("Web App Root directory ["+webRootDirName+"] not found.");
			return;
		}
		File webInfDir = new File(webRootDir, "WEB-INF");
		if(!webInfDir.exists())
		{
			System.err.println("Warning: Directory ["+webRootDirName+"] doesn't look like a Web App Root directory (no WEB-INF found).");
		}
		
		System.out.println("Validating open SUIT XML: "+xmlFileName);
		
    	String urlStr = "file:/"+(xmlFile.getAbsolutePath().replace('\\', '/'));
		URL url = null;
        try
        {
//   System.out.println("url: "+urlStr);
	        url = new URL(urlStr);
        }
        catch(MalformedURLException e)
        {
    		System.err.println("Error while building xml file url ["+urlStr+"].");
	        e.printStackTrace(System.err);
	        return;
        }
        try
        {
        	XmlInstantiator doc = instantiate(url, webRootDir);

    		System.out.println(" --> XML Model instantiation done.");
    		System.out.println("Dumping messages ("+(dumpLevel == IInitializationSupport.ERROR ? "error" : (dumpLevel == IInitializationSupport.WARNING ? "warning" : "info"))+"):");
			doc.dumpMessages(dumpLevel, new PrintWriter(System.out));
        }
        catch(Exception e)
        {
    		System.err.println("Error while instantiating xml model.");
	        e.printStackTrace(System.err);
	        return;
        }
	}
	public static XmlInstantiator instantiate(URL xmlFile, File webRootDir) throws IOException, SAXException, ParserConfigurationException, FactoryConfigurationError
	{
		IValidationSupport validSupport = new J2EEValidator(webRootDir);
		return XmlInstantiator.instantiate(xmlFile, validSupport);
	}
	// =====================================================================================
	// === J2EE Validator
	// =====================================================================================
	public static class J2EEValidator implements IValidationSupport
	{
		private ServletContext _ctx;
		
		public J2EEValidator(File iWebRoot)
		{
			_ctx = new J2EEServletContext(iWebRoot);
		}
		public ServletContext getServletContext()
		{
			return _ctx;
		}
		public boolean checkResource(String iResourcePath)
		{
			if(_ctx == null)
				return true;
			if(iResourcePath == null)
			    return true;
			if(!iResourcePath.startsWith("/"))
			    iResourcePath = "/"+iResourcePath;
//			return _ctx.getResourceAsStream(iResourcePath) != null;
            try {
                return _ctx.getResource(iResourcePath) != null;
            } catch (MalformedURLException e) {
                return false;
            }
		}
	}
	// =====================================================================================
	// === J2EE Servlet Context
	// =====================================================================================
	public static class J2EEServletContext implements ServletContext
	{
		File webRootDir;
		Hashtable<String, Object> attributes = new Hashtable<String, Object>();
		
		public J2EEServletContext(File webRootDir)
		{
			this.webRootDir = webRootDir;
		}

		public Object getAttribute(String name)
		{
			return attributes.get(name);
		}

		public Enumeration getAttributeNames()
		{
			return attributes.keys();
		}
		public void removeAttribute(String name)
		{
			attributes.remove(name);
		}

		public void setAttribute(String name, Object value)
		{
			attributes.put(name, value);
		}

		public ServletContext getContext(String name)
		{
			return null;
		}

		public String getInitParameter(String name)
		{
			// --- servlet init parameters are passed as Java Properties:
			// -Dservlet.<name>=<value>
			return System.getProperty("servlet."+name);
		}

		public Enumeration getInitParameterNames()
		{
			return null;
		}

		public int getMajorVersion()
		{
			return 0;
		}

		public String getMimeType(String arg0)
		{
			return null;
		}

		public int getMinorVersion()
		{
			return 0;
		}

		public RequestDispatcher getNamedDispatcher(String arg0)
		{
			return null;
		}

		public RequestDispatcher getRequestDispatcher(String arg0)
		{
			// TODO Auto-generated method stub
			return null;
		}

		public String getRealPath(String path)
		{
			if(File.separatorChar == '\\')
				path = path.replace('/', '\\');
			String realPath = webRootDir.getAbsolutePath()+path;
//			System.out.println("getRealPath('"+path+"') --> "+realPath+" ("+(File.separatorChar == '\\' ? "win" : "unix")+")");
			return realPath;
		}

		public URL getResource(String path) throws MalformedURLException
		{
			File f = new File(getRealPath(path));
			if(f.exists())
				return f.toURL();
			return null;
		}

		public InputStream getResourceAsStream(String path)
		{
			try
			{
				return new FileInputStream(getRealPath(path));
			}
			catch (FileNotFoundException e)
			{
				return null;
			}
		}

		public Set getResourcePaths(String path)
		{
			URL url = null;
			try
			{
				url = getResource(path);
			}
			catch (MalformedURLException e)
			{
			}
			if(url == null)
				return null;
			Set ret = new HashSet();
			ret.add(url);
			return ret;
		}

		public String getServerInfo()
		{
			// TODO Auto-generated method stub
			return null;
		}

		public Servlet getServlet(String arg0) throws ServletException
		{
			// TODO Auto-generated method stub
			return null;
		}

		public String getServletContextName()
		{
			// TODO Auto-generated method stub
			return null;
		}

		public Enumeration getServletNames()
		{
			// TODO Auto-generated method stub
			return null;
		}

		public Enumeration getServlets()
		{
			// TODO Auto-generated method stub
			return null;
		}

		public void log(Exception arg0, String arg1)
		{
			System.out.println("J2EEServletContext - log: "+arg1);
			arg0.printStackTrace();
		}

		public void log(String arg0, Throwable arg1)
		{
			System.out.println("J2EEServletContext - log: "+arg0);
			arg1.printStackTrace();
		}

		public void log(String arg0)
		{
			System.out.println("J2EEServletContext - log: "+arg0);
		}

	}
}
