/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.session;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.core.impl.context.OpenSuitSessionImpl;
import org.ow2.opensuit.xml.base.page.ctx.IPageContextDef;


/**
 * Holds the page context.
 */
public abstract class PageContext
{
	private IPageContextDef contextManager;
	private PageContext parent;
	private HashMap<String, Object> _name2Attr = new HashMap<String, Object>();
	private List<IPageContextListener> _listeners;

	public PageContext(IPageContextDef contextManager, PageContext parent)
	{
		this.contextManager = contextManager;
		this.parent = parent;
	}
	public PageContext getParent()
	{
		return parent;
	}
	
	/**
	 * Retrieves an attribute attached to this page context.
	 * @param iName Attribute name.
	 * @return Attribute value.
	 */
	public Object getAttribute(String iName)
	{
		Object o = _name2Attr.get(iName);
		if(o != null)
			return o;
		if(parent != null)
			return parent.getAttribute(iName);
		return null;
	}
	/**
	 * Returns all attribute names.
	 * @return
	 */
	public Set<String> getAttributeNames()
	{
		// TODO: return also parent attribute names?
	    return _name2Attr.keySet();
	}
	/**
	 * Removes the attribute with given name.
	 * @param iName Attribute name to remove.
	 */
	public void removeAttribute(String iName)
	{
		_name2Attr.remove(iName);
		/* TODO: parent ?
		if(_name2Attr.remove(iName) == null && parent != null)
			parent.removeAttribute(iName);
		*/
	}
	/**
	 * Sets an attribute attached to this page context.
	 * This data is cached in session as long as the page lives.
	 * @param iName Attribute name.
	 * @param iValue Attribute value.
	 */
	public void setAttribute(String iName, Object iValue)
	{
		_name2Attr.put(iName, iValue);
	}

	/**
	 * Returns the page context manager responsible for this page context.
	 * @return
	 */
	public IPageContextDef getContextManager()
	{
		return contextManager;
	}
	/**
	 * Registers a listener to this page context.
	 * @param listener
	 */
	public void addListener(IPageContextListener listener)
	{
		if(_listeners == null)
			_listeners = new ArrayList<IPageContextListener>(1);
		_listeners.add(listener);
	}
	/**
	 * Sets this page context as the current one in the open SUIT session.
	 * @param request
	 * @throws Exception
	 */
	public void setCurrent(HttpServletRequest request) throws Exception
	{
		OpenSuitSessionImpl.getSession(request).setCurrentPageContext(this);
	}
	/**
	 * Loads this page context.
	 * This will possibly trigger actions.
	 * @param request
	 * @throws Exception
	 */
	public void load(HttpServletRequest request) throws Exception
	{
		// --- make this context the active one
		OpenSuitSessionImpl.getSession(request).setCurrentPageContext(this);
		
		// --- then load
		try
		{
			doLoad(request);
		}
		catch(Exception e)
		{
			// --- unset this context as current
			OpenSuitSessionImpl.getSession(request).setCurrentPageContext(null);
			throw e;
		}
	}
	/**
	 * Quits this page context.
	 * This will possibly trigger actions.
	 * @param request
	 * @throws Exception
	 */
	public void quit(HttpServletRequest request) throws Exception
	{
		// --- make this context the active one
		OpenSuitSessionImpl.getSession(request).setCurrentPageContext(this);
		
		// --- trigger listener(s)
		if(_listeners != null)
		{
			for(IPageContextListener listener : _listeners)
				listener.onQuit(request, this);
		}
		
		// --- then quit
		doQuit(request);
		
		// --- and finally make parent context the active one (may be null)
		OpenSuitSessionImpl.getSession(request).setCurrentPageContext(parent);
	}
	
	protected abstract void doLoad(HttpServletRequest request) throws Exception;
	
	protected abstract void doQuit(HttpServletRequest request) throws Exception;
}
