/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.form.field;
import java.io.PrintWriter;
import java.lang.reflect.Type;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.opensuit.core.session.OpenSuitSession;
import org.ow2.opensuit.core.util.BaseMessages;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.core.util.UrlBuilder;
import org.ow2.opensuit.core.validation.LocalizedValidationError;
import org.ow2.opensuit.core.validation.ValidationError;
import org.ow2.opensuit.xml.base.Application;
import org.ow2.opensuit.xml.base.enums.DateType;
import org.ow2.opensuit.xml.base.html.CssStyleSheet;
import org.ow2.opensuit.xml.base.html.IFrame;
import org.ow2.opensuit.xml.base.html.form.BaseForm;
import org.ow2.opensuit.xml.base.html.form.FormRenderingContext;
import org.ow2.opensuit.xml.base.html.form.FormSubmitContext;
import org.ow2.opensuit.xml.base.page.IPage;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.annotations.XmlRoot;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;



@XmlDoc("A date form field.<br/>" +
		"Supported value types are: java.util.Date or java.util.Calendar")
@XmlElement
public class DateField extends BaseSingleValueField implements IInitializable
{
	private static Log logger = LogFactory.getLog(DateField.class);
	
	@XmlRoot
	protected Application application;
	
	@XmlAncestor
	protected IPage iPage;
	
	@XmlAncestor
	protected BaseForm baseForm;
	
	@XmlDoc("The date display style. Default: DateOnly.")
	@XmlAttribute(name="Type", required=false)
	private DateType type= DateType.DateOnly;
	
	private Class<?> bindingType;
	
	public void initialize(IInitializationSupport initSupport, Object additionalSupport)
	{
		super.initialize(initSupport, additionalSupport);
		
		if(initSupport.initialize(bind))
		{
			// --- register this component as a request handler
			application.registerRequestHandler(getForm().getPathID()+"."+getFieldName(), this);
//		    if(application.registerRequestHandler(getForm().getPathID()+"."+getFieldName(), this) != null)
//	        	initSupport.addValidationMessage(this, "ID", IInitializationSupport.INFO, "Callback URLs may not be stable on this component. To solve this specify a unique ID in the page.");
		    
	        // --- check binding type
			bindingType = bind.getType();
			
			// --- check binding type
	    	if(!bind.isConvertible(Date.class))
	    	{
				initSupport.addValidationMessage(this, "Bind", IInitializationSupport.ERROR, "Expression "+bind+" has invalid type ("+bind.getType()+"): Date or Calendar expected.");
	    	}
	    	else
	    	{
				// --- check validator types
				if(validators != null)
				{
					for(int i=0; i<validators.length; i++)
					{
						if(!validators[i].isTypeValid(bindingType))
							initSupport.addValidationMessage(this, "Validators", IInitializationSupport.ERROR, "Validator "+i+" ("+validators[i]+") does not support binding type ("+bind.getType()+").");
					}
				}
	    	}
		}
	}
	public Type getValueGenericType()
	{
		return getValueType();
	}
	public Class<?> getValueType()
	{
		return bindingType;
	}
	@Override
	protected Object getValueFromRequest(HttpServletRequest iRequest, FormSubmitContext submitContext) throws ValidationError
	{
		Date date = null;
		
		// --- 1: parse date
		if(type == DateType.DateAndTime || type == DateType.DateOnly)
		{
			String val = submitContext.getSubmitValue(getFieldName());
			if(val == null)
				return null;
			val = val.trim();
			if(val.length() == 0)
				return null;
			
			// --- a date was entered: parse
			OpenSuitSession session = OpenSuitSession.getSession(iRequest);
			DateFormat format = DateType.getInputFormat(session.getLocaleConfig(), DateType.DateOnly);
			try
	        {
				date = format.parse(val);
	        }
	        catch(ParseException e)
	        {
	        	throw new LocalizedValidationError("validation.date.wrong_format", "opensuit", new Object[]{format instanceof SimpleDateFormat ? ((SimpleDateFormat)format).toPattern() : format.toString()});
	        }
		}
		
		// --- 2: parse time
		if(type == DateType.DateAndTime || type == DateType.TimeOnly)
		{
			int hours = Integer.parseInt(submitContext.getSubmitValue(getFieldName()+"_h"));
			int minutes = Integer.parseInt(submitContext.getSubmitValue(getFieldName()+"_m"));
			int seconds = Integer.parseInt(submitContext.getSubmitValue(getFieldName()+"_s"));
			
			if(date == null)
				date = new Date();
			
			date.setHours(hours);
			date.setMinutes(minutes);
			date.setSeconds(seconds);
		}
		
//        return Expression.EXPRESSION_FACTORY.getConverter().convert(date, type)ReflectionHelper.date2Obj(date, bind.getType());
		return date;
	}
	public void preRender(HttpServletRequest iRequest) throws Exception
	{
	}
	protected void renderInput(HttpServletRequest iRequest, HttpServletResponse iResponse, FormRenderingContext iContext) throws Exception
	{
		PrintWriter iWriter = iResponse.getWriter();
		boolean editable = iContext.isEditable() && isEditable(iRequest);
		OpenSuitSession session = OpenSuitSession.getSession(iRequest);
		
		// --- rcupration de la valeur
		Date dateVal = null;
		String date = null;
		int hours = 0;
		int minutes = 0;
		int seconds = 0;
//		String val = null;
		if(editable && iContext.getSubmitContext() != null && iContext.getSubmitContext().getValidationErrors() != null)
		{
			// --- after a submit: retrieve value from the request
			if(type == DateType.DateAndTime || type == DateType.DateOnly)
			{
				date = iContext.getSubmitContext().getSubmitValue(getFieldName());
			}
			if(type == DateType.DateAndTime || type == DateType.TimeOnly)
			{
				hours = Integer.parseInt(iContext.getSubmitContext().getSubmitValue(getFieldName()+"_h"));
				minutes = Integer.parseInt(iContext.getSubmitContext().getSubmitValue(getFieldName()+"_m"));
				seconds = Integer.parseInt(iContext.getSubmitContext().getSubmitValue(getFieldName()+"_s"));
			}
		}
		else
		{
			// --- first display: retireve value from the bean
			dateVal = bind.invoke(iRequest, Date.class);
			if(dateVal != null)
			{
				if(type == DateType.DateAndTime || type == DateType.DateOnly)
				{
					DateFormat format = DateType.getInputFormat(session.getLocaleConfig(), DateType.DateOnly);
					date = format.format(dateVal);
				}
				if(type == DateType.DateAndTime || type == DateType.TimeOnly)
				{
					hours = dateVal.getHours();
					minutes = dateVal.getMinutes();
					seconds = dateVal.getSeconds();
				}
			}
		}
		if(date == null)
			// --- intialisation  vide
			date = "";
		
		if(editable)
		{
			if(iContext.isEditMode())
				HtmlUtils.includeBaseJavaScript(iRequest, iResponse, "Form.js");
			
			// --- render field(s)
			if(type == DateType.DateAndTime || type == DateType.DateOnly)
			{
				// --- render date field
				iWriter.print("<input type='text'");
				iWriter.print(" id='");
				iWriter.print(getFieldId());
				iWriter.print("'");
				iWriter.print(" name='");
				iWriter.print(getFieldName());
				iWriter.print("'");
				iWriter.print(" size='10'");
				if(!iContext.isEditMode())
					iWriter.print(" disabled='disabled'");
				iWriter.print(" value=\"");
				iWriter.print(HtmlUtils.encode2HTML(date));
				iWriter.print("\"");
				iWriter.print(">");
				
				// --- render the calendar link
				if(iContext.isEditMode())
				{
					String calendarText = BaseMessages.getMessage(iResponse.getLocale(), "datefield.show_calendar");
					UrlBuilder calendarUrl = application.createHandlerUrl(iRequest, this, "renderCalendar");
					iWriter.print("<a class='CalendarButton'");
					/*
					iWriter.print(" href='");
					iWriter.print(calendarUrl.toUrl(session.getLocaleConfig().getCharSet(), true));
					iWriter.print("'");
					iWriter.print(" target='_blank'");
					*/
					
					iWriter.print(" href='javascript:void(0)'");
					iWriter.print(" onclick=\"");
					iWriter.print("DateField_showCalendar(event, '"+baseForm.getId()+"', '"+getFieldName()+"', '"+calendarUrl.toUrl(session.getLocaleConfig().getCharSet(), true)+"'); return false;");
					iWriter.print("\"");
					iWriter.print(" title=\"");
					iWriter.print(HtmlUtils.encode2HTML(calendarText));
					iWriter.print("\"");
					iWriter.print(">");
					iWriter.print(HtmlUtils.encode2HTML(calendarText));
					iWriter.print("</a>");
				}
			}
			if(type == DateType.DateAndTime || type == DateType.TimeOnly)
			{
				// --- render time fields
				
				// --- hour
				iWriter.print(HtmlUtils.encode2HTML(BaseMessages.getMessage(iResponse.getLocale(), "datefield.hours")));
				iWriter.print("<select name='");
				iWriter.print(getFieldName());
				iWriter.print("_h'");
				if(!iContext.isEditMode())
					iWriter.print(" disabled='disabled'");
				iWriter.println(">");
				for(int i=0; i<24; i++)
				{
					iWriter.print("<option value='");
					iWriter.print(String.valueOf(i));
					iWriter.print("'");
					if(i==hours)
						iWriter.print(" selected='selected'");
					iWriter.print(">");
					iWriter.print(String.valueOf(i));
					iWriter.println("</option>");
				}
				iWriter.println("</select>");
				
				// --- minutes
				iWriter.print(HtmlUtils.encode2HTML(BaseMessages.getMessage(iResponse.getLocale(), "datefield.minutes")));
				iWriter.print("<select name='");
				iWriter.print(getFieldName());
				iWriter.print("_m'");
				if(!iContext.isEditMode())
					iWriter.print(" disabled='disabled'");
				iWriter.println(">");
				for(int i=0; i<60; i++)
				{
					iWriter.print("<option value='");
					iWriter.print(String.valueOf(i));
					iWriter.print("'");
					if(i==minutes)
						iWriter.print(" selected='selected'");
					iWriter.print(">");
					iWriter.print(String.valueOf(i));
					iWriter.println("</option>");
				}
				iWriter.println("</select>");
				
				// --- seconds
				iWriter.print(HtmlUtils.encode2HTML(BaseMessages.getMessage(iResponse.getLocale(), "datefield.seconds")));
				iWriter.print("<select name='");
				iWriter.print(getFieldName());
				iWriter.print("_s'");
				if(!iContext.isEditMode())
					iWriter.print(" disabled='disabled'");
				iWriter.println(">");
				for(int i=0; i<60; i++)
				{
					iWriter.print("<option value='");
					iWriter.print(String.valueOf(i));
					iWriter.print("'");
					if(i==seconds)
						iWriter.print(" selected='selected'");
					iWriter.print(">");
					iWriter.print(String.valueOf(i));
					iWriter.println("</option>");
				}
				iWriter.println("</select>");
			}
		}
		else
		{
			// --- display value
			if(dateVal != null)
			{
				DateFormat format = DateType.getInputFormat(session.getLocaleConfig(), type);
				iWriter.print(HtmlUtils.encode2HTML(format.format(dateVal)));
			}
		}
	}
	// ================================================================================================
	// === render calendar methods
	// ================================================================================================
	private static HashMap<String, String> key2Name = new HashMap<String, String>();
	
	private static String getMonthAndYearName(Locale locale, Calendar cal)
	{
		return getMonthName(locale, cal.get(Calendar.DAY_OF_MONTH))+" "+cal.get(Calendar.YEAR);
	}
	
	private static String getMonthName(Locale locale, int month)
	{
		String key = "m."+locale.getLanguage()+"."+month;
		String name = key2Name.get(key);
		if(name == null)
		{
			synchronized(key2Name)
            {
				// --- fullname
				SimpleDateFormat sdf = new SimpleDateFormat("MMMMM", locale);
//				Date d = new Date(0, month, 1);
				Calendar cal = Calendar.getInstance().getInstance(locale);
				cal.set(Calendar.MONTH, month);
				name = sdf.format(cal.getTime());
				key2Name.put(key, name);
            }
		}
		return name;
	}
	private static String getDayName(Locale locale, int day)
	{
		String key = "d."+locale.getLanguage()+"."+day;
		String name = key2Name.get(key);
		if(name == null)
		{
			synchronized(key2Name)
            {
				// --- fullname
//				SimpleDateFormat sdf = new SimpleDateFormat("EEEEE", locale);
				// --- 3 characters
				SimpleDateFormat sdf = new SimpleDateFormat("E", locale);
//				Date d = new Date(0, 0, day);
				Calendar cal = Calendar.getInstance(locale);
				cal.set(Calendar.DAY_OF_WEEK, day);
				name = sdf.format(cal.getTime());
				key2Name.put(key, name);
            }
		}
		return name;
	}
	public static void main(String[] args)
	{
		Locale[] locales = {Locale.FRENCH, Locale.US, Locale.UK};
		System.out.println(" saturday: "+Calendar.SATURDAY);
		System.out.println(" sunday: "+Calendar.SUNDAY);
		System.out.println(" monday: "+Calendar.MONDAY);
		for(int i=0; i<locales.length; i++)
		{
			Calendar cal = Calendar.getInstance(locales[i]);
			System.out.println("Locale "+locales[i]);
			
			System.out.println(" first day in week: "+cal.getFirstDayOfWeek()+" ("+getDayName(locales[i], cal.getFirstDayOfWeek())+")");
			
			/*
			for(int j=0; j<12; j++)
				System.out.println(" month "+j+": "+getMonthName(locales[i], j));
			
			for(int j=0; j<7; j++)
				System.out.println(" day "+j+": "+getDayName(locales[i], j));
			*/
			
			int minWeek = cal.getActualMinimum(Calendar.WEEK_OF_MONTH);
			int maxWeek = cal.getActualMaximum(Calendar.WEEK_OF_MONTH);
			System.out.println("Min / max week of this month: "+minWeek+"/"+maxWeek);
			
			cal.set(Calendar.DAY_OF_MONTH, 1);
			System.out.println("first day of this month is: "+getDayOfWeek(cal.get(Calendar.DAY_OF_WEEK)));
			minWeek = cal.get(Calendar.WEEK_OF_YEAR);
			int lastDayOfMonth = cal.getActualMaximum(Calendar.DAY_OF_MONTH);
			cal.set(Calendar.DAY_OF_MONTH, lastDayOfMonth);
			maxWeek = cal.get(Calendar.WEEK_OF_YEAR);
			System.out.println("Min / max week of this year: "+minWeek+"/"+maxWeek+" ("+(maxWeek-minWeek+1)+" weeks)");
			
			for(int d=1; d<=lastDayOfMonth; d++)
			{
				cal.set(Calendar.DAY_OF_MONTH, d);
				System.out.println("Day "+d+" is week "+cal.get(Calendar.WEEK_OF_YEAR));
			}
		}
	}
	private static String getDayOfWeek(int dayOfWeek)
	{
		switch(dayOfWeek)
		{
			case Calendar.MONDAY:
				return "monday";
			case Calendar.TUESDAY:
				return "tuesday";
			case Calendar.WEDNESDAY:
				return "wednesday";
			case Calendar.THURSDAY:
				return "thrusday";
			case Calendar.FRIDAY:
				return "friday";
			case Calendar.SATURDAY:
				return "saturday";
			case Calendar.SUNDAY:
				return "sunday";
			default:
				return "??? ("+dayOfWeek+")";
		}
	}
	public void renderCalendar(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		logger.debug("renderCalendar(): "+iRequest.getQueryString());
		
		// --- 1: retrieve selected date
		OpenSuitSession session = OpenSuitSession.getSession(iRequest);
		DateFormat inputFormat = DateType.getInputFormat(session.getLocaleConfig(), DateType.DateOnly);
		String selectedStr = iRequest.getParameter("selected");
		Date selected = null;
		if(selectedStr != null)
		{
			selectedStr = selectedStr.trim();
			// --- a date was entered: parse
			try
	        {
				selected = inputFormat.parse(selectedStr);
	        }
	        catch(ParseException e)
	        {
	        	// --- ignore
	        }
		}
		
		// --- retrieve month to display
//		Date displayedMonth = new Date();
		int month = 0;
		int year = 0;
		String monthStr = iRequest.getParameter("month");
		if(monthStr != null)
		{
			// --- displayed mont specified: get
			int idx = monthStr.indexOf('-');
			month = Integer.parseInt(monthStr.substring(0, idx))-1;
			year = Integer.parseInt(monthStr.substring(idx+1));
		}
		else if(selected != null)
		{
			// --- display month of selected date
			month = selected.getMonth();
			year = selected.getYear()+1900;
		}
		else
		{
			// --- get current month
			Calendar displayedMonth = Calendar.getInstance(iResponse.getLocale());
			month = displayedMonth.get(Calendar.MONTH);
			year = displayedMonth.get(Calendar.YEAR);
		}
		
		// --- render
		Calendar cal = Calendar.getInstance(iResponse.getLocale());
		cal.set(year, month, 1, 0, 0, 0);
		
		UrlBuilder url;
		
		DateFormat monthUrlFormat = new SimpleDateFormat("M-yyyy");
		DateFormat monthDisplayFormat = new SimpleDateFormat("MMMMM yyyy", iResponse.getLocale());
		
		// --- 2: render calendar page
		// ---------------------------
		iResponse.setContentType("text/html;charset="+session.getLocaleConfig().getCharSet());
		PrintWriter iWriter = iResponse.getWriter();
		
		// --- 3: render
		//TODO: other headers
		
		iWriter.println(HtmlUtils.getHTML401TransitionalDoctype());
		iWriter.print("<html lang=\"");
		iWriter.print(session.getLocaleConfig().getLocale().getLanguage());
		iWriter.println("\">");
		iWriter.println("<head>");

		// --- write page title
		iWriter.print("<title>");
		iWriter.print(HtmlUtils.encode2HTML(BaseMessages.getMessage(iResponse.getLocale(), "calendar.title", new Object[]{monthDisplayFormat.format(cal.getTime())})));
		iWriter.println("</title>");

		// --- for modal dialog
		iWriter.println("<base target='_self'>");
		
		// --- meta tags
		// html content type
		iWriter.println("<meta http-equiv='content-type' content='text/html;charset="+session.getLocaleConfig().getCharSet()+"'>");
		// no cache (for all browsers)
		// et pourquoi pas ???
		iWriter.println("<meta http-equiv='cache-control' content='no-cache'>");
		iWriter.println("<meta http-equiv='pragma' content='no-cache'>");
		iWriter.println("<meta http-equiv='expires' content='0'>");
		/*
		// --- les liens relatifs le sont par rapport  l'application
		iWriter.print("<base href='");
		iWriter.print(iRequest.getScheme());
		iWriter.print("://");
		iWriter.print(iRequest.getServerName());
		iWriter.print(":");
		iWriter.print(String.valueOf(iRequest.getServerPort()));
		iWriter.print(iRequest.getContextPath());
		iWriter.print("/");
		iWriter.println("'>");
		*/

		// --- import stylesheets
		IFrame frame = application.getFrame(iPage.getFrameName());
		CssStyleSheet[] styleSheets = frame.getStyleSheets();
		if(styleSheets != null)
		{
			for(int i=0; i<styleSheets.length; i++)
			{
				iWriter.print("<link rel='stylesheet' type='text/css' href='");
				iWriter.print(styleSheets[i].getHref(iRequest));
				iWriter.println("'>");
			}
		}
		
		// --- include javascript
		HtmlUtils.includeBaseJavaScript(iRequest, iResponse, "Form.js");

		iWriter.println("</head>");
		
		iWriter.println("<body class=Calendar>");
		
		iWriter.println("<table class=Calendar>");
		
		// --- toolbar row
		// ---------------
		iWriter.print("<tr class=Toolbar>");
		iWriter.print("<th colspan=8>");
		
		iWriter.print("<div>");
		
		// --- prev year link
		cal.set(year, month, 1);
		cal.add(Calendar.YEAR, -1);
		url = application.createHandlerUrl(iRequest, this, "renderCalendar");
		url.setParameter("month", monthUrlFormat.format(cal.getTime()));
		if(selected != null)
			url.setParameter("selected", selectedStr);
		iWriter.print("<a class='PrevYear' href='");
		iWriter.print(url.toUrl(session.getLocaleConfig().getCharSet(), true));
		iWriter.print("' title='");
		iWriter.print(monthDisplayFormat.format(cal.getTime()));
		iWriter.print("'>&nbsp;</a>");
		
		iWriter.print("&nbsp;");

		// --- prev month link
		cal.set(year, month, 1);
		cal.add(Calendar.MONTH, -1);
		url = application.createHandlerUrl(iRequest, this, "renderCalendar");
		url.setParameter("month", monthUrlFormat.format(cal.getTime()));
		if(selected != null)
			url.setParameter("selected", selectedStr);
		iWriter.print("<a class='PrevMonth' href='");
		iWriter.print(url.toUrl(session.getLocaleConfig().getCharSet(), true));
		iWriter.print("' title='");
		iWriter.print(monthDisplayFormat.format(cal.getTime()));
		iWriter.print("'>&nbsp;</a>");
		
		iWriter.print("&nbsp;");
		
		// --- write current month
		cal.set(year, month, 1);
		iWriter.print("<span class='Title'>");
		iWriter.print(monthDisplayFormat.format(cal.getTime()));
		iWriter.print("</span>");
		
		iWriter.print("&nbsp;");

		// --- next month link
		cal.set(year, month, 1);
		cal.add(Calendar.MONTH, +1);
		url = application.createHandlerUrl(iRequest, this, "renderCalendar");
		url.setParameter("month", monthUrlFormat.format(cal.getTime()));
		if(selected != null)
			url.setParameter("selected", selectedStr);
		iWriter.print("<a class='NextMonth' href='");
		iWriter.print(url.toUrl(session.getLocaleConfig().getCharSet(), true));
		iWriter.print("' title='");
		iWriter.print(monthDisplayFormat.format(cal.getTime()));
		iWriter.print("'>&nbsp;</a>");
		
		iWriter.print("&nbsp;");
		
		// --- next year link
		cal.set(year, month, 1);
		cal.add(Calendar.YEAR, +1);
		url = application.createHandlerUrl(iRequest, this, "renderCalendar");
		url.setParameter("month", monthUrlFormat.format(cal.getTime()));
		if(selected != null)
			url.setParameter("selected", selectedStr);
		iWriter.print("<a class='NextYear' href='");
		iWriter.print(url.toUrl(session.getLocaleConfig().getCharSet(), true));
		iWriter.print("' title='");
		iWriter.print(monthDisplayFormat.format(cal.getTime()));
		iWriter.print("'>&nbsp;</a>");
		
		iWriter.print("</div>");

		iWriter.print("</th>");
		iWriter.println("</tr>");
		
		// --- header row (day names)
		// --------------------------
		cal.set(year, month, 1);
		iWriter.print("<tr class=DayNames>");
		
		iWriter.print("<th scope=col class=Week>");
		iWriter.print(HtmlUtils.encode2HTML(BaseMessages.getMessage(iResponse.getLocale(), "calendar.week")));
		iWriter.print("</th>");
		
		int firstDayInWeek = cal.getFirstDayOfWeek();
		int day = firstDayInWeek;
		for(int i=0; i<7; i++)
		{
			iWriter.print("<th scope=col>");
			iWriter.print(getDayName(iResponse.getLocale(), day));
			iWriter.print("</th>");
			day++;
			if(day > 6) day = 0;
		}
		iWriter.println("</tr>");
		
		// --- week rows
		// -------------
		cal.set(year, month, 1);
		
		cal.set(Calendar.DAY_OF_MONTH, 1);
		int firstWeek = cal.get(Calendar.WEEK_OF_YEAR);
		int lastDayOfMonth = cal.getActualMaximum(Calendar.DAY_OF_MONTH);
		cal.set(Calendar.DAY_OF_MONTH, lastDayOfMonth);
		int lastWeek = cal.get(Calendar.WEEK_OF_YEAR);
//		int nbWeeks = maxWeek-minWeek+1;
		
		cal.set(Calendar.WEEK_OF_YEAR, firstWeek);
//		int nbWeeks = cal.getActualMaximum(Calendar.WEEK_OF_MONTH);
//		cal.set(Calendar.WEEK_OF_MONTH, 1);
		cal.set(Calendar.DAY_OF_WEEK, firstDayInWeek);
		
//		for(int i=0; i<nbWeeks; i++)
		boolean lastWeekRendered = false;
		while(!lastWeekRendered)
		{
			if(cal.get(Calendar.WEEK_OF_YEAR) == lastWeek)
				// --- we're about to render last week
				lastWeekRendered = true;

			iWriter.print("<tr class=Week>");
			
			iWriter.print("<td class=Week>");
			iWriter.print("<span>");
			iWriter.print(String.valueOf(cal.get(Calendar.WEEK_OF_YEAR)));
			iWriter.print("</span>");
			iWriter.print("</td>");
			
			for(int j=0; j<7; j++)
			{
				int dayInWeek = cal.get(Calendar.DAY_OF_WEEK);
				Date dayDate = cal.getTime();

				iWriter.print("<td class=Day>");
				iWriter.print("<a class='Day");
				if(dayInWeek == Calendar.SATURDAY || dayInWeek == Calendar.SUNDAY)
					iWriter.print(" WeekEnd");
				if(cal.get(Calendar.MONTH) != month)
					iWriter.print(" OutMonth");
				if(selected != null && selected.getYear() == dayDate.getYear() && selected.getMonth() == dayDate.getMonth() && selected.getDate() == dayDate.getDate())
					iWriter.print(" Selected");
				iWriter.print("'");
				iWriter.print(" href='javascript:void(0)'");
				iWriter.print(" onclick=\"");
				iWriter.print("DateField_setDateAndClose('"+baseForm.getId()+"', '"+getFieldName()+"', '"+inputFormat.format(dayDate)+"'); return false;");
				iWriter.print("\"");
//				iWriter.print(" href=\"javascript:void(");
//				iWriter.print("DateField_setDateAndClose('"+baseForm.getID()+"', '"+getFieldName()+"', '"+inputFormat.format(dayDate)+"')");
//				iWriter.print(")\"");
				iWriter.print(">");
				iWriter.print(String.valueOf(cal.get(Calendar.DATE)));
				iWriter.print("</a>");
				iWriter.print("</td>");
				
				// --- add one day
				cal.add(Calendar.DAY_OF_YEAR, 1);
			}
			iWriter.println("</tr>");
		}
		
		iWriter.println("</table>");
		
		iWriter.println("</body>");
		iWriter.println("</html>");
		
		iWriter.flush();
		iWriter.close();
	}
}
